#include <iostream>
#include <sstream>
#include <list>
#include <algorithm>

using namespace std;

//-----------------------------------------------------------
typedef list<float> pipe;

ostream& operator <<( ostream& out, const pipe& obj ) {
   out << "{ ";
   for( auto i = obj.begin(); i != obj.end(); i++ ) {
      auto j = i;
      j++;
      out << *i << ( j == obj.end() ? "" : ", " ); 
   }
   return out << " }";
}

istream& operator >>( istream& inp, pipe& obj ) {
   obj.clear();
   string e;
   while( true ) {
      getline( inp, e );
      if( '#' == e[ 0 ] ) continue;  // it's a comment
      if( inp.rdstate() & ios::eofbit ) break;
      if( 0 == e.length() ) break;   // empty line
      istringstream ist( e );
      string s;
      while( ist >> s ) {
         float d = 0.;   
         try { d = stof( s ); }   
         catch( exception const& e ) {
            goto err;
         }
         if( d <= 0.0 ) goto err;
         obj.push_back( d );
      }
   }
   return inp;
err:
   cout << "illegal input: error!" << endl;
   exit( 1 );
}

float max( const pipe& obj ) {
   float ret = 0.;
   for( auto i = obj.begin(); i != obj.end(); i++ ) 
      if( *i > ret ) ret = *i;
   return ret;
}

float summa( const pipe& obj ) {
   float ret = 0.;
   for( auto i = obj.begin(); i != obj.end(); i++ ) 
      ret += *i;
   return ret;
}

//-----------------------------------------------------------

class engine {
public: 
   static const unsigned numb = 2;
   pipe line[ numb ];
   engine( void ) {}
   engine( pipe& p ) {
      line[ 0 ].clear();
      line[ 1 ] = p;
   }
   engine( const engine& e ) {
      for( unsigned i = 0; i < numb; i++ )
         line[ i ] = e.line[ i ];
   }
   ~engine( void ) { 
      for( unsigned i = 0; i < numb; i++ ) 
         line[ i ].clear(); 
   }
   engine& operator =( const engine& e ) {
      for( unsigned i = 0; i < numb; i++ ) {
         line[ i ].clear(); 
         line[ i ] = e.line[ i ];
      }
      return *this;
   }
   pipe& operator []( int row ) { return line[ row ]; }
   bool move( float val, unsigned to, unsigned from = numb - 1 ) {
      list<float>::iterator i = find( line[ from ].begin(), line[ from ].end(), val );   
      if( i == line[ from ].end() ) return false;
      line[ to ].push_back( val );
      line[ from ].erase( i );
      return true;
   }
   float scattering( void ) {
      float s1 = 0., s2 = 0.;
      for( unsigned i = 0; i < numb; i++ ) {
         float s = summa( line[ i ] );
         s1 += s;
         s2 += s * s;
      }
      s1 /= numb;
      s2 = s2 / numb - s1 * s1;
      return s2;
   }
   friend ostream& operator <<( ostream& out, const engine& obj ) {
      for( unsigned i = 0; i < obj.numb; i++ )
         out <<  char( 'A' + i ) << ":" <<  obj.line[ i ] << " "; // << endl;
      return out;
   }
};

//-----------------------------------------------------------

float limit;
engine optim;
bool debug;

unsigned long long balancing( engine& e, pipe prev ) {
   static string level;
   const string shift( "   " );
   if( debug ) level += shift;
   unsigned long long variants = 1;
   float s = e.scattering();
   if( debug ) cout << level << e << " => " << s << endl;
   if( s < limit ) {
      limit = s;
      optim = e;
   }
   pipe p = e[ 1 ];
   if( p.size() > 1 )
      for( auto i = p.begin(); i != p.end(); i++ ) {
         if( find( prev.begin(), prev.end(), *i ) == prev.end() ) {
            engine e1( e );
            e1.move( *i, 0 );
            variants += balancing( e1, prev );
         }
         prev.push_back( *i );
      }
   if( debug ) level = level.substr( 0, level.size() - shift.size() );
   return variants;
}

int main( int argc, char *argv[] ) {
   debug = argc > 1 && "debug" == string( argv[ 1 ] );
   cout << "enter a sequence of numbers in a line (empty line - EOF)" << endl;
   pipe P;
   cin >> P;
   cout << P << endl 
        << "the maximum number is " << max( P ) << endl
        << "the total summa is " << summa( P ) << endl;
/*   pipe P1;
   cout << P1 << endl; 
   P1 = P;
   cout << P1 << endl; */
   engine E( P );
/* engine E1;
   cout << E << endl << "scattering=" << E.scattering() << endl;
   E1 = E;
   cout << E1 << endl << "scattering=" << E1.scattering() << endl; */
/*   E1.move( 393, 0 ); */
/*   E1.move( 11, 0 ), E1.move( 12, 0 );
   cout << E1 << endl << "scattering=" << E1.scattering() << endl; */
   optim = E;
   limit = E.scattering();
   unsigned long long variants = balancing( E, pipe() );
   cout << "the best option is: " << optim << endl
        << "scattering=" << limit << endl
        << "the total number of combinations " << variants << endl;
}
