#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <climits>
using namespace std;

#ifdef DEBUG
   bool debug = true;
#else
   bool debug = false;
#endif

typedef vector<bool> line;
ostream& operator <<( ostream& out, line& obj ) {
   for( unsigned i = 0; i < obj.size(); i++ ) 
      out << ( obj[ i ] ? 1 : 0 ) << ( i == obj.size() - 1 ? "" : ", " );
   return out;
}

class matrix {    // класс матриц
private:
   typedef vector<line> table;
   vector< vector<bool> > *data;
   bool ok;
   unsigned rows, // размер 1-й индекс Y
            cols; // размер 2-й индекс X
   void clean( void ) {
      for( unsigned r = 0; r < rows; r++ ) (*data)[ r ].clear();
      (*data).clear();
   }
public:
   bool inline good( void ) { return ok; }
   const int col( void ) { return cols; }
   const int row( void ) { return rows; }
   matrix( void ) {
      ok = true;
      cols = rows = 0;
      data = new table();      
   }
   matrix( const matrix& m ) {
      if( !( ok = m.ok ) ) return;
      data = new table();      
      cols = m.cols;
      rows = m.rows;
      for( unsigned r = 0; r < rows; r++ )
         data->push_back( (*m.data)[ r ] );
   }
   ~matrix( void ) {
      clean();
      delete data;
   }
   line& operator []( int row ) { return (*data)[ row ]; }
   inline void clear( void ) {
      clean();
      cols = rows = 0;
   }
   friend ostream& operator <<( ostream& out, matrix& obj ) {
      if( !obj.ok ) 
         return out << "destroyed object" << endl;
      for( unsigned r = 0; r < obj.rows; r++ ) 
         out << (*(obj.data))[ r ] << endl; 
      return out; 
   }
   friend istream& operator >>( istream& inp, matrix& obj );
};

istream& operator >>( istream& inp, matrix& obj ) {     // ввод матрицы
   obj.clear();
   int r = 0;
   while( true ) {
      string e;
      getline( inp, e );
      if( inp.rdstate() & ios::eofbit || e.empty() ) break;
      istringstream ist( e );
      int n = 0;
      line l;
      int d;
      while( true ) {
         ist >> d;
         n++;
         l.push_back( d != 0 );
         if( ist.rdstate() & ios::eofbit ) break;
      }
      if( 0 == r ) obj.cols = l.size();
      else if( obj.cols != l.size() ) {
         obj.ok = false;
         return inp;
      }
      obj.data->push_back( l );
      r++;
   }
   obj.rows = r;
   obj.ok = true;
   return inp;
};

int read_size( istream& inp ) {                         // ввод размера матрицы
   string e;
   int n = 0;
   while( 0 == n ) {
      getline( inp, e );
      if( e.empty() ) continue;
      istringstream ist( e );
      while( true ) { 
         int d = 0;
         ist >> d;
         if( ist.rdstate() & ios::failbit ) {
            return -2;
         }
         n++;
         if( n > 1 ) return -1;
         if( ist.rdstate() & ios::eofbit ) return d;
      }
   }
   return 0;
};

int way( matrix& m, int y, int x ) {
   if( y == m.row() - 1 && x == m.col() - 1 ) return 1; // это конечная точка
   matrix m1( m );
   m1[ y ][ x ] = false;                                // отметить как пройдено
   int y1, x1, res = INT_MAX;
   for( int i = 0; i < 4; i++ ) {                       // обойти 4 соседа
      switch( i ) {
         case 0:
            y1 = y; x1 = x + 1; break;
         case 1:
            y1 = y; x1 = x - 1; break;
         case 2:
            y1 = y + 1; x1 = x; break;
         case 3:
            y1 = y - 1; x1 = x; break;
      }
      if( x1 < 0 || x1 >= m.col() ||
          y1 < 0 || y1 >= m.row() ) continue;          // поле за пределами матрицы
      if( !m1[ y1 ][ x1 ] ) continue;                  // недопустиммый шаг
      int ret = way( m1, y1, x1 );
      if( ret > 0 && ret < res ) 
         res = ret + 1;
   }
   return res == INT_MAX ? 0 : res;
}

int main( int argc, char *argv[] ) {
   ifstream fin;
   fin.open( argc > 1 ? argv[ 1 ] : "input.txt" );
   if( !fin ) {                                        // ошибочный файл
      cerr << "wrong input file" << endl;
      return 1;
   }
   int n = read_size( fin );
   if( n <= 0 ) {
      cerr << "matrix size error" << endl;
      return 1;
   } 
   if( debug ) cout << "matrix size " << n << " x " << n << endl;
   matrix M;
   fin >> M;                                           // заполнение матрицы из файла
   if( !M.good() || ( M.row() != n ) || ( M.row() != M.col() ) || 
       !M[ 0 ][ 0 ] || !M[ M.row() - 1 ][ M.col() - 1 ] ) {
      cerr << "wrong matrix data" << endl;
      return 1;
   }
   if( debug ) cout << "matrix M[" << M.row() << "," << M.col() << "]=" << endl << M;
   fin.close();
   int w = way( M, 0, 0 );
   w = w > 0 ? w - 1 : 0;
   cout << "trace in " << w << " steps" << endl;
   ofstream fou;
   fou.open( argc > 2 ? argv[ 2 ] : "output.txt" );     
   if( !fou ) {                                        // ошибочный файл
      cerr << "wrong output file" << endl;
      return 1;
   } 
   fou << w << endl;
   fou.close();
}
