#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <list>
#include <climits>
using namespace std;

#ifdef DEBUG
   bool debug = true;
#else
   bool debug = false;
#endif

typedef vector<bool> line;
ostream& operator <<( ostream& out, line& obj ) {
   for( unsigned i = 0; i < obj.size(); i++ ) 
      out << ( obj[ i ] ? 1 : 0 ) << ( i == obj.size() - 1 ? "" : ", " );
   return out;
}

struct coord {
   int y, x;
   coord( int y, int x ) {
      this->y = y;
      this->x = x;
   }
   coord operator +( coord& s ) {
      return coord( y + s.y, x + s.x );
   };
   coord& operator +=( const coord & s ) {
      y += s.y;
      x += s.x;
      return *this;
   };
   friend ostream& operator <<( ostream& out, coord& obj ) {
      return out << '<' << obj.y << ',' << obj.x << '>';
   }
};


class matrix {    // класс матриц логических клеток
private:
   typedef vector<line> table;
   vector< vector<bool> > *data;
   bool ok;
   unsigned rows, // размер 1-й индекс Y
            cols; // размер 2-й индекс X
   void clean( void ) {
      for( unsigned r = 0; r < rows; r++ ) (*data)[ r ].clear();
      (*data).clear();
   }
public:
   bool inline good( void ) { return ok; }
   const int col( void ) { return cols; }
   const int row( void ) { return rows; }
   matrix( void ) {
      ok = true;
      cols = rows = 0;
      data = new table();      
   }
   matrix( const matrix& m ) {
      if( !( ok = m.ok ) ) return;
      data = new table();      
      cols = m.cols;
      rows = m.rows;
      for( unsigned r = 0; r < rows; r++ )
         data->push_back( (*m.data)[ r ] );
   }
   ~matrix( void ) {
      clean();
      delete data;
   }
   line& operator []( int row ) { return (*data)[ row ]; }
   inline void clear( void ) {
      clean();
      cols = rows = 0;
   }
   friend ostream& operator <<( ostream& out, matrix& obj ) {
      if( !obj.ok ) 
         return out << "destroyed object" << endl;
      for( unsigned r = 0; r < obj.rows; r++ ) 
         out << (*(obj.data))[ r ] << endl; 
      return out; 
   }
   friend istream& operator >>( istream& inp, matrix& obj );
};

istream& operator >>( istream& inp, matrix& obj ) {     // ввод матрицы из файла
   obj.clear();
   int r = 0;
   while( true ) {
      string e;
      getline( inp, e );
      if( inp.rdstate() & ios::eofbit || e.empty() ) break;
      istringstream ist( e );
      int n = 0;
      line l;
      int d;
      while( true ) {
         ist >> d;
         n++;
         l.push_back( d != 0 );
         if( ist.rdstate() & ios::eofbit ) break;
      }
      if( 0 == r ) obj.cols = l.size();
      else if( obj.cols != l.size() ) {
         obj.ok = false;
         return inp;
      }
      obj.data->push_back( l );
      r++;
   }
   obj.rows = r;
   obj.ok = true;
   return inp;
};

int read_size( istream& inp ) {                         // ввод размера матрицы
   string e;
   int n = 0;
   while( 0 == n ) {
      getline( inp, e );
      if( e.empty() ) continue;
      istringstream ist( e );
      while( true ) { 
         int d = 0;
         ist >> d;
         if( ist.rdstate() & ios::failbit ) {
            return -2;
         }
         n++;
         if( n > 1 ) return -1;
         if( ist.rdstate() & ios::eofbit ) return d;
      }
   }
   return 0;
};

list<coord> way( matrix& m, coord& p ) {
   list<coord> v;                                              // пустой список
   if( p.y == m.row() - 1 && p.x == m.col() - 1 ) {            // это конечная точка
      v.push_back( p );
      return v; 
   }
   matrix m1( m );
   m1[ p.y ][ p.x ] = false;                                   // отметить как пройдено
   unsigned len = INT_MAX;
   static coord neib[] = { coord( 0, 1 ), coord( 0, -1 ),
                           coord( 1, 0 ), coord( -1, 0 ) };    // соседи
   for( unsigned i = 0; i < sizeof( neib ) / sizeof( neib[ 0 ] ); i++ ) {  
      coord p1 = p + neib[ i ];
      if( p1.x < 0 || p1.x >= m.col() ||
          p1.y < 0 || p1.y >= m.row() ) continue;              // поле за пределами матрицы
      if( !m1[ p1.y ][ p1.x ] ) continue;                      // недопустиммый шаг
      list<coord> v1 = way( m1, p1 );                          // последующая трасса
      if( v1.size() > 0 && v1.size() < len ) {                 // это одно из решений
         v1.push_front( p );
         len = v1.size(); 
         v = v1;
      }
   }
   return v; 
}

ostream& operator <<( ostream& out, list<coord>& obj ) {       // вывод трассы обхода
   if( obj.empty() )
      out << "<>";
   for( list<coord>::iterator i = obj.begin(); i != obj.end(); i++ )
      out << *i << ' ';
   return out;
}

int main( int argc, char *argv[] ) {
   ifstream fin;
   fin.open( argc > 1 ? argv[ 1 ] : "input.txt" );
   if( !fin ) {                                                // ошибочный файл
      cerr << "wrong input file" << endl;
      return 1;
   }
   int n = read_size( fin );
   if( n <= 0 ) {
      cerr << "matrix size error" << endl;
      return 1;
   } 
   if( debug ) cout << "matrix size " << n << " x " << n << endl;
   matrix M;
   fin >> M;                                                   // заполнение матрицы из файла
   if( !M.good() || ( M.row() != n ) || ( M.row() != M.col() ) || 
       !M[ 0 ][ 0 ] || !M[ M.row() - 1 ][ M.col() - 1 ] ) {
      cerr << "wrong matrix data" << endl;
      return 1;
   }
   if( debug ) cout << "space matrix M[" << M.row() << "," << M.col() << "]=" << endl << M;
   fin.close();
   coord beg( 0, 0 );
   list<coord> trace = way( M, beg );                          // поиск кратчайшей трассы
   int steps = trace.empty() ? 0 : trace.size() - 1;
   if( debug ) cout << "trace in " << steps << " steps" << endl << trace << endl;
   ofstream fou;
   fou.open( argc > 2 ? argv[ 2 ] : "output.txt" );     
   if( !fou ) {                                                // ошибочный файл
      cerr << "wrong output file" << endl;
      return 1;
   } 
   fou << steps << endl;
   fou.close();
}


